from .attributes import AttributeCollection
from .subcorpora import SubcorpusCollection


class CorpusCollection:
    def __init__(self, client):
        self.client = client

    def get(self, name):
        return Corpus(self.client, name)

    def list(self):
        return [Corpus(self.client, corpus) for corpus in
                self.client.api.corpus_list_coprora()]


class Corpus:
    def __init__(self, client, name):
        self.client = client
        self.name = name
        self.size = client.api.cl_attribute_size('{}.word'.format(name))
        # self.info = client.api.corpus_info(name)
        self.charset = client.api.corpus_charset(name)
        # self.full_name = client.api.corpus_full_name(name)
        self.properties = client.api.corpus_properties(name)
        self.attributes = AttributeCollection(client, self)
        self.subcorpora = SubcorpusCollection(client, self)

    def lookups_by_cpos(self, cpos_list):
        cpos_list = list(set(cpos_list))
        lookups = {}
        if cpos_list:
            lookups['cpos_lookup'] = {}
        for cpos in cpos_list:
            lookups['cpos_lookup'][cpos] = {}
        for attr in self.attributes.positional.list():
            cpos_attr_values = attr.values_by_cpos(cpos_list)
            for i, cpos in enumerate(cpos_list):
                lookups['cpos_lookup'][cpos][attr.name] = cpos_attr_values[i]
        for attr in self.attributes.structural.list():
            if attr.has_values:
                continue
            cpos_attr_ids = attr.ids_by_cpos(cpos_list)
            for i, cpos in enumerate(cpos_list):
                if cpos_attr_ids[i] != -1:
                    lookups['cpos_lookup'][cpos][attr.name] = cpos_attr_ids[i]
            occured_attr_ids = list(filter(lambda x: x != -1,
                                           set(cpos_attr_ids)))
            if not occured_attr_ids:
                continue
            subattrs = \
                self.attributes.structural.list(filters={'part_of': attr})
            if not subattrs:
                continue
            lookup_name = '{}_lookup'.format(attr.name)
            lookups[lookup_name] = {}
            for attr_id in occured_attr_ids:
                lookups[lookup_name][attr_id] = {}
            for subattr in subattrs:
                subattr_values = subattr.values_by_ids(occured_attr_ids)
                for i, subattr_value in enumerate(subattr_values):
                    lookups[lookup_name][occured_attr_ids[i]][subattr.name] = \
                        subattr_value
        return lookups

    def query(self, query, subcorpus_name='Results'):
        self.client.api.cqp_query(self.name, subcorpus_name, query)
        return self.subcorpora.get('Results')
