from ..constants import (CONST_FIELD_KEYWORD, CONST_FIELD_MATCH,
                         CONST_FIELD_MATCHEND, CONST_FIELD_TARGET)


class SubcorpusCollection:
    def __init__(self, client, corpus):
        self.client = client
        self.corpus = corpus

    def get(self, name):
        return Subcorpus(self.client, self.corpus, name)

    def list(self):
        return [Subcorpus(self.client, self.corpus, subcorpus) for subcorpus in
                self.client.api.cqp_list_subcorpora(self.corpus.name)]


class Subcorpus:
    def __init__(self, client, corpus, name):
        self.client = client
        self.corpus = corpus
        self.name = name
        self._name = '{}:{}'.format(corpus.name, name)
        self.fields = {}
        if client.api.cqp_subcorpus_has_field(self._name, CONST_FIELD_MATCH):
            self.fields['match'] = CONST_FIELD_MATCH
        if client.api.cqp_subcorpus_has_field(self._name,
                                              CONST_FIELD_MATCHEND):
            self.fields['matchend'] = CONST_FIELD_MATCHEND
        if client.api.cqp_subcorpus_has_field(self._name, CONST_FIELD_TARGET):
            self.fields['target'] = CONST_FIELD_TARGET
        if client.api.cqp_subcorpus_has_field(self._name, CONST_FIELD_KEYWORD):
            self.fields['keyword'] = CONST_FIELD_KEYWORD
        self.size = client.api.cqp_subcorpus_size(self._name)

    def drop(self):
        return self.client.api.cqp_drop_subcorpus(self._name)

    def dump(self, field, first, last):
        return self.client.api.cqp_dump_subcorpus(self._name, field, first,
                                                  last)

    def export(self, context=25, cutoff=float('inf'), expand_lists=False,
               offset=0):
        if self.size == 0:
            return {"matches": []}
        first_match = max(0, offset)
        last_match = min((offset + cutoff - 1), (self.size - 1))
        match_boundaries = \
            zip(self.dump(self.fields['match'], first_match, last_match),
                self.dump(self.fields['matchend'], first_match, last_match))
        cpos_list = []
        matches = []
        for match_start, match_end in match_boundaries:
            c = (match_start, match_end)
            lc = rc = None
            if context == 0:
                cpos_list += list(range(match_start, (match_end + 1)))
            else:
                lc_rbound = max(0, (match_start - 1))
                if lc_rbound != match_start:
                    lc_lbound = max(0, (match_start - 1 - context))
                    lc = (lc_lbound, lc_rbound)
                    cpos_list_lbound = lc_lbound
                else:
                    cpos_list_lbound = match_start
                rc_lbound = min((match_end + 1), (self.corpus.size - 1))
                if rc_lbound != match_end:
                    rc_rbound = min((match_end + 1 + context),
                                    (self.corpus.size - 1))
                    rc = (rc_lbound, rc_rbound)
                    cpos_list_rbound = rc_rbound
                else:
                    cpos_list_rbound = match_end
                cpos_list += list(range(cpos_list_lbound,
                                        (cpos_list_rbound + 1)))
            if expand_lists:
                match = {'lc': list(range(lc[0], (lc[1] + 1))),
                         'c': list(range(c[0], (c[1] + 1))),
                         'rc': list(range(rc[0], (rc[1] + 1)))}
            else:
                match = {'lc': lc, 'c': c, 'rc': rc}
            matches.append(match)
        lookups = self.corpus.lookups_by_cpos(cpos_list)
        return {'matches': matches, **lookups}

    def fdist_1(self, cutoff, field, attribute):
        return self.client.api.cqp_fdist_1(self._name, cutoff,
                                           field, attribute._name)

    def fdist_2(self, cutoff, field_1, attribute_1, field_2, attribute_2):
        return self.client.api.cqp_fdist_2(self._name, cutoff,
                                           field_1, attribute_1._name,
                                           field_2, attribute_2._name)
